#!/usr/bin/env python
# -*- coding: utf-8 -*-
#

#Listing of all the Nahuatl codes, letters of the alphabet, etc.
# To be included in NahTab2MultLexc.py.


#----------------------------XML Tags------------------------
#Lexeme tag (surrounds the entire lexeme, including any or all of the following tags):
sLexemeTag = 'refGroup'

#For the remaining tags, we have (at least potentially) more than one tag
# of a given type, e.g. \lxoa_pr and \lxoa.  So the following definitions
# each have lists of tags.  Within each list, the order of elements corresponds
# to the preference, e.g. [lxoa_pr lxoa ] means that we will take the <lxoa_pr>
# element if it is present, else the <lxoa> element.  These are lists
# (rather than tuples) only because that allows us to concantenate all of them,
# and extract from each lex entry something we can use to create an error msg.

#Target language word: either <lxoa_pr> or <lxoa>, with former preferred.
# Some lex entries in the dictionary have citation forms for only other dialects;
# these will generate a warning (but not a very informative one, since no
# citation form will be shown in the warning msg).
sCitationFormTags = ['lxoa_pr', 'lxoa']
##FIX: all lxoa_alt's should also be loaded in addition to the lxoa field, iff there is no lxoa_pr.
# May be multiple lxoa_pr fields.
# Always ignore the lxoa_c field (see Jonathan's email of 28 Apr 07 5:31 PM).

#POS: <psm>
sPOSTags = ['psm']

#Subclass (may be empty): <infn> (for nouns only). The \infn field marks a subclass
# of nouns, with each subclass taking different affixes.  The subclasses are
# therefore stored in different files, like N^N1, N^N2 (based on the subclass
# N1, N2 etc.).
sSubclassTags = ['infn']


#Allomorph classes.
# Verbs have a large number of these, for stem allomorphs, listed in the \infv field.
# (This is quite different from the \infn field used for noun subclasses.)
sStemClassTags = ['infv']
# For nouns, the conceptually similar field is the \allomorph field:
sAllomorphTags = ['allomorph']

# Nouns have two paradigm (declension) classes, of which only one--for Spanish loans--is marked,
# by the presence of a \loan field.  (The \loan field gives the Spanish word from which the Nahuatl
# loanword comes.  We ignore the contents, and assume that if there's a \loan field, it's from Spanish.)
# Since these have different \infn fields (usually, but not always, either N1 or N1=N2), we treat
# this as a declension difference, not a POS subclass difference.  They will be marked by the flag
# diacritic '@P.STRATUM.SPANISH@'.
sParadigmTags = ['loan']


#Irregularity tags
# Nouns often have irregular or absent plurals (or else they are only plural, never singular).
# This is encoded in the \plural field, with a number of variants.  For irregular plurals,
# a typical SFM field looks like this:
#    \plural Irregular: <kam>tla:la:jes</kam>
## There is quite a bit of variation, including in the choice of embedded XML tags; see my email
## of 28 Apr 07 10:23 PM asking about this.
sIrregularTags = ['plural']

#----------------------------Alphabet------------------------
#A list of chars which are legal in the Nahuatl alphabet (some of them are
# only used in Spanish loans):
sAlphabet = ( 'a',  u'á',  'b',  'c', 'd', 'e', u'é', 'f', 'g',
              'h',   'i',  u'í', 'j', 'k', 'l',  'm',  'n', 'o',
             u'ó',  'p',   'r',  's', 't', 'u', u'ú', 'w', 'x',
              'y', ':'
            )
  #If some of these chars look odd, you may need to re-open this file in a UTF-8 encoding.
  #No 'qvz' (or space, which would be used in phrasal entries, or before a comment).
  #Does the accented actually 'u' appear?
  #These are just the chars as Unicode defines them--e.g. there is not a special case
  # for 'ch' or the long vowels as units.
sCharactersToStrip = " \t-"
  #Set of characters that it's safe to strip from right and left of \lx.. fields

#-----------------------------POSs----------------------------
#The commented-out POSs are those we don't yet know how to deal with.
# See my 2 Apr 06 email to Jonathan.
dictPOSNames = { #This maps names of POSs in the dictionary to a canonical name;
                 # it also contains derived POSs which are not in the dictionary,
                 # but are created by NormalizeParadigmClass() in NahTab2MultLexc.py.
                 "Adj":                            "Adj"
               , "Adj(ap)":                        "Adj"
               , "Adv":                            "Adv"
               , "Adv-interog":                    "Adv-interog"
               , "Adv-manner":                     "Adv-manner"
               , "Adv-place":                      "Adv-place"
               , "Adv-time":                       "Adv-time"
#               , "Associated motion ending":       "Associated"  #Used in dict, but not needed in xfst
                                                                  # because it's an affix
               , "Aux":                            "V0"           #Conjugates like V0 (see Jonathan's
                                                                  # email of 16 Nov 06 11:21 AM)
               , "Baby":                           "Baby"
               , "Call":                           "Call"
               , "Clit":                           "Clit"       #Most of these are irrelevent, but there's
                                                                # one which looks like a postposition
               , "Conj":                           "Conj"
               , "Coord":                          "Coord"
               , "Dem-adj":                        "Dem-adj"
               , "Dir":                            "Dir"
               , "Discourse":                      "Discourse"
#               , "Durative":                       "Durative"
               , "Funct":                          "Funct"
               , "Interj":                         "Interj"
               , "Loc":                            "Loc"
               , "Modal":                          "Modal"
               , "N":                              "N"
               #Map all additional types to 'N' for now (these types are indicated by 'N(X)',
               # where 'X' is some code):
               , "N(Proper)":                      "N"
                    #See also "N(proper)" below
               , "N(address)":                     "N"
               , "N(ag)":                          "N"
               , "N(ap)":                          "N"
               , "N(attributive)":                 "N"
               , "N(complex)":                     "N"
               , "N(complex/final)":               "N"
               , "N(complex/initial)":             "N"
               , "N(greeting)":                    "N"
               , "N(loc)":                         "N"
               , "N(proper)":                      "N"
               , "N(rel)":                         "N"

               , "N-N":                            "N-N"
               , "N-ad":                           "N-ad"
               , "N-compl":                        "N-compl"
               , "N-tsi:n":                        "N-tsi"
               , "N/Adj":                          "N"
               , "N1":                             "N1"
               , "N2":                             "N2"
               , "N; Adj":                         "N"
               , "N; Interj":                      "N"
               , "N^N1":                           "N^N1"             #Derived POS
               , "N^N2":                           "N^N2"             #Derived POS
               , "N^N1dom":                        "N^N1dom"          #Derived POS
               , "N^N1N2":                         "N^N1N2"           #Derived POS
               , "N^Inalienable":                  "N^Inalienable"    #Derived POS
               , "N^PartWhole":                    "N^PartWhole"      #Derived POS
               , "N^PartWholeOnly":                "N^PartWholeOnly"  #Derived POS
               , "N^Undetermined":                 "N^Undetermined"   #Derived POS (will probably go away eventually)
               , "N^ObligPoss":                    "N^ObligPoss"      #Derived POS
               , "Neg":                            "Neg"
#               , "Nomlz":                          "Nomlz"
#               , "Noun(rel)":                      "Noun"
               , "Num":                            "Num"
               , "PM":                             "PM"
               , "Pr":                             "Pr"
               , "Pr(dem)":                        "Pr"          #These additional types of pronouns are just
                                                                 # mapped to 'Pr' for now
               , "Pr(demon)":                      "Pr"
               , "Pr(ind)":                        "Pr"
               , "Pr(indef)":                      "Pr"
               , "Pr(int)":                        "Pr"
               , "Pr(obj)":                        "Pr"
               , "Pr(obj); Pr(poss)":              "Pr"
               , "Pr(pos)":                        "Pr"
               , "Pr/Adj(dem)":                    "Pr"
               , "Pr/Adj(demon)":                  "Pr"
               , "Pr?":                            "Pr"
#               , "Pref(obj)":                      "Pref"
#               , "Pref(pos)":                      "Pref"
#               , "Pref(sub)":                      "Pref"
#               , "Prefix":                         "Prefix"
               , "Pronoun":                        "Pronoun"    #Should be mapped to 'Pr'?
               , "Quant":                          "Quant"
               , "Reltz":                          "Reltz"
               , "Reltzer":                        "Reltzer"    #Should be mapped to 'Reltz'?
#               , "Stat":                           "Stat"
               , "Subord":                         "Subord"
#               , "Suf":                            "Suf"
#               , "Suf(adj)":                       "Suf"
#               , "Suf(loc)":                       "Suf"
#               , "Suf-poss":                       "Suf-poss"
               , "Undeter":                        "Undeter"
               , "V0":                             "V0"
               , "V0(cop)":                        "V0cop"        #Copular "verbs" are NOT treated as verbs here, so they don't need
                                                                  # an allomorph class (below), even though the dictionary defines one
                                                                  # ("\infv Copular").
               , "V1":                             "V1"
               , "V1(refl/only)":                  "V1"           #This distinction needs to be made
               , "V2":                             "V2"
               , "V2(pass)":                       "V2"           #This distinction needs to be made
               , "V2(refl/dif)":                   "V2"           #This distinction needs to be made
               , "V2(refl/only)":                  "V2"           #This distinction needs to be made
               , "V3":                             "V3"
               , "V3(refl)":                       "V3"           #This distinction needs to be made
               , "V4":                             "V4"
               }

#---------------POS hierarchy:
sPOSSuperclass = {#Nouns (most of the ff. noun codes are composites of "N" and the (thus far known) subclasses.
                  # The subclasses (the parts to the right of the "^") are defined in dictNounSubclasses.
                  # Any new codes in this list will need to be added to dictPOSNames as well.
                  # There are lots of other noun-like POSs (see below, sPOSs), but I'm not sure what
                  # their codes mean.  See my email of 9 Sept 06 11:45 PM.
                   "N":               "N"   #Degenerate case needed when we haven't yet determined subclass
                  ,"N^N1":            "N"
                  ,"N^N2":            "N"
                  ,"N^N1dom":         "N"
                  ,"N^N1N2":          "N"
                  ,"N^Inalienable":   "N"
                  ,"N^PartWhole":     "N"
                  ,"N^PartWholeOnly": "N"
                  ,"N^ObligPoss":     "N"
                  #Inflected verbs (not including V0cop):
                  ,"V0":              "V"
                  ,"V1":              "V"
                  ,"V2":              "V"
                  ,"V3":              "V"
                  ,"V4":              "V"
                  #Adjectives (not all are inflected):
                  ,"Adj":             "Adj"
                 }
#---------------Inflected POSs:
sInflectedCategories = sPOSSuperclass.keys()
  #The words of these categories have paradigms in which at least one form is inflected.
  # This will become more complicated if we define subclasses of POSs that aren't inflected
  # (like adverbs or pronouns).  V0cop is treated with other UNinflected words.

#-----------------Stem Classes---------------------
#For Nahuatl verbs, there are multiple stem classes.  These are in addition to other allomorphy
# codes found on both nouns and verbs.  They are found in the \infv field.
dictStemClasses = {"V":  {"class-1":                                 "^C1"

                         ,"class-2a":                                "^C2a"
                         ,"class-2a(Oa)":                            "^C2a"

                         ,"class-2b":                                "^C2b"
                         ,"class-2b(Oa)":                            "^C2b"

                         ,"class-3a":                                "^C3a"
                         ,"class-3a(ch)":                            "^C3a"
                         ,"class-3a(k)":                             "^C3a"
                         ,"class-3a(kw)":                            "^C3a"
                         ,"class-3a(m)":                             "^C3a"
                         ,"class-3a(ts)":                            "^C3a"
                         ,"class-3a(w)":                             "^C3a"
                         ,"class-3a(y)":                             "^C3a"

                         ,"class-3b(ow)":                            "^C3b"

                         ,"class-3c":                                "^C3c"
                         ,"class-3c(chia)":                          "^C3c"
                         ,"class-3c(pia)":                           "^C3c"

                         ,"class-3d(ti)":                            "^C3d"

                         ,"class-4a":                                "^C4a"
                         ,"class-4a(oni)":                           "^C4a"
                         ,"class-4a(pass)":                          "^C4a"

                         ,"class-4b(ya-s)":                          "^C4b"
                         ,"class-4b(ya-x)":                          "^C4b"

                         ,"class-4c":                                "^C4c"
                         ,"class-4c(pano)":                          "^C4c"

                         ,"class-4d(tia)":                           "^C4d"

                         ,"Irregular; see <klamoa>ita</klamoa>":     "^C4ita"
                                   #These behave like "ordinary class 4 verbs" (IAW Jonathan's Bug Box)
                                   # except for the durative stem.  They all end in /ita/ or /ata/
                                   # (or accented versions of these).
                         ,"Irregular; see <klamoa>maka</klamoa>":    "^C4maka"
                         ,"Irregular; see <klamoa>mati</klamoa>":    "^Cmati"
                         ,"Irregular; see <klamoa>wa:hlaw</klamoa>": "^Caw"
                                   #The unique verb 'wa:hlaw'
                         ,"Irregular; see <klamoa>yaw</klamoa>":     "^Caw"
                                   #The unique verb 'yaw'
                         }
                  }
#Following lists verb stem class codes found in \infv fields which we do not (yet)
# map over to xfst codes.
#   "Bound morpheme: see <klamoa>oni</klamoa>and <klamoa>tlai</klamoa>" #   Listed as V2, but bound
#   "Copular"  #   Used only with copular verbs, which are not treated as verbs here
   ##FIX: The negative copular verb 'xa:k' is listed as psm = V1, infv = Copular;
   # need special case for it
#   "Durative":
#   "Durative(refl)":
#   "Irregular":
#   "Irregular (Am); see <klam>kukwa</klam>":
#   "Irregular in Ameyaltepec: <kam>ne:chichitia</kam> , <kam>mitsitsitia</kam>, <kam>kititia</kam> , <kam>te:chichitia</kam> , <kam>me:chichitia</kam> ,":
#   "Irregular: Stems: <kam>kukwa</kam>; <kam>koko:</kam>; <kam>kokoh</kam>":
#   "Irregular: perfective <kam>o:kipopo:w</kam> ; future <kam>kipupwas</kam>; prog. <kam>kipopo:wtok</kam> ; dir. <kam>ma nikpupwati</kam>":
#   "Irregular: perfective: <koa>o:k:p:w</koa>; future <koa>k:pows</koa>; progressive <koa>k:po:htok</koa>; directional <koa>ma n:hpowti</koa>":
#   "Irregular; <kamoa>-tikateh</kamoa>":
#   "Irregular; Perfective <kam>o:kikit</kam> / <kam>o:kikitik</kam>":
#   "Irregular; see <klam>kukwa</klam>":
#   "Irregular; see <klam>kukwa</klam> (Am) <klam>pupwa</klam> (Am) / <kloa>popowa</kloa>(Oa)":
#   "Irregular; see <klam>kukwa</klam> (Am), <kloa>kokowa</kloa>(Oa); Irregular; see <klam>kukwa</klam> (Am), <kloa>kokowa</kloa>(Oa); Irregular; see":
#   "Irregular; see <klam>pukwa</klam>":
#   "Irregular; see <klamoa>-tiw</klamoa>":
#   "Irregular; see <klamoa>kah</klamoa>":
#   "Irregular; see <kloa>popowa</kloa>":
#   "No inflections":
#   "Pending (inflectional paradigm still not determined)":
#   "class-4a/associated motion"   #Not used with verbal PSMs
##FIX:
  # However,
  # there needs to be special treatment for one class, 4a(oni) (see below)


#-----------------Allomorph Classes----------------
#Following gives the mapping for allomorphy classes from a normalized
# version of the dictionary code in the \allomorph field to the xfst code.
# Note that a number of dictionary codes for verbs are matched to a single
# xfst code, since we ignore the parenthesized material (which for
# the most part concerns phonologically predictable differences).
# See the comments following this defn for codes that we do not (yet) map.
# NB: if codes are added here, they need to be added to PhonologyClasses.xfst
# as well, so they can be deleted at the appropriate point in the derivation.
#                       POS             Dict Code                   xfst code
#                       ---             ---------                   ---------
dictAllomorphClasses = {"N":           {"x=ch":                     "^x=ch"
                                       ,"s=ts":                     "^s=ts"
                                       ,"h=k" :                     "^h=k"
                                       ,"h=w" :                     "^h=w"
                                       ,"h=l" :                     "^h=l"
                                       ,"0=h" :                     "^0=h"
                                       ,"y-deletion" :              "^yDeletion"
                                       ,"Gender":                   "^gender"  #Note the case shift!
                                       ,"i-epen":                   "^iEpen"
                                       ,"Vowel-stem loss":          "^VLoss"
                                       }
                       ,"V":           {"i-epen"   :                "^iEpen"
                                       ,"Object deletion":          "^objDel"
                                       }
##FIX: Found this here, but it's unclear what it was supposed to do.
                       ,"Uninflected": {"Copular":                  ""  #Dummy: V0(cop) gets mapped to POS of 'Uninflected',
                                       }                                # and has \infv Copular, which we ignore.
                       }




#----------------Subclasses-------------------
# Subclasses of the Noun POS, i.e. things like obligatorily possessed vs. optionally possessed.
# Ff. comes from Jonathan's '14 Possessed nouns.doc'; the codes in the right-hand column
# are used in the \infn field but with any space characters removed.
# For purposes of xfst, we attach these codes to the bare 'N'.  These will wind up in
# different files, and we can decide what to do with them in the xfst code.  (For example,
# N1(dom) might go with the N1 nouns for the generator, but with the N2 nouns for the parser.)
# The codes in the second column are intended to serve as valid file names, in conjunction with
# a POS prefix of "N".
# ****IF ANY CODES ARE ADDED HERE, the corresponding compound noun names
# ****(like "N^N1") should be added to the list sNounPOSs.
#                 POS   Dict Code        : xfst/ Grammar Code
#                 ---   ---------          ------------------
dictSubclasses = {"N": {"N1"              : "^N1"             #Cannot be possessed
                       ,"N1(dom)"         : "^N1dom"          #Rarely if ever possessed (for semantic reasons)
                       ,"N2"              : "^N2"             #May be unpossessed or possessed (alienable)
#                       ,"N2(rdp.s/h)"  : "TBD"               #May be possessed or unpossessed, but when possessed is always alienable.
                                                              #However, the (rdp.s/h) code indicates that the first syllable is
                                                              # a reduplicant with short vowel and thus when a possessor marker
                                                              # has a short vowel, it is lengthened instead of the overt reduplicant
                                                              #    koko:hli  has N2(rdp.s/h)
                                                              # and thus the possessed forms are
                                                              #   no:ko:l
                                                              #   mo:ko:l
                                                              #   i:koko:l
                                                              #etc.
                                                              ##FIX: Added in 5 Mar 2008 doc < Jonathan, not added here yet.
                       ,"N1=N2"           : "^N1N2"           #Stem is unchanged between unpossessed and possessed;
                                                              # dictionary also has "N1/2", unclear if same
#                       ,N1(dom)=N2	      : "TBD"             #Stem is unchanged between unpossessed and possessed
                                                              # (when possessed is always alienable) but possessed form
                                                              # is rare.
                                                              ##FIX: Added in 5 Mar 2008 doc < Jonathan, not added here yet.
                       ,"Inalienable"     : "^Inalienable"    #Always possessed
                       ,"Part/whole"      : "^PartWhole"      #May be possessed part/whole (-yo) or alienable
#                       ,"Part/whole(rare)"   : "TBD" 	      #May be unpossessed or possessed but when possessed may be either
                                                              # alienable or part/whole.  However, the part/whole possession is
                                                              # documented though rare (e.g., tlake:ntli, cloth or clothes;
                                                              # cf. notlake:n 'my clothes'. Also documented is i:tlake:nyo
                                                              # 'its cloth covering, e.g, a dusk jacket on a book).
                                                              ##FIX: Added in 5 Mar 2008 doc < Jonathan, not added here yet.
                       ,"Part/whole(only)": "^PartWholeOnly"  #May be possessed only as part/whole
                       ,"Obligposs"       : "^ObligPoss"      #Obligatorily possessed (i.e., headword is possessed stem)
                                                              # There is usually a space char between "Oblig" and "poss",
                                                              # but that is stripped out by ModPOSwithSubclass()
#                       ,Irregular  : "TBD" 	              #Explained in separate document
                                                              ##FIX: Added in 5 Mar 2008 doc < Jonathan, not added here yet.
                       ,"Undetermined"    : "^N1"             #Information lacking: do not generate possessed forms
                                                              # (currently treated as uninflected, hence mapped to ^N1)
                       }
                 }
  #In addition, there are a number of noun codes that appear in the \infn fields of
  # the dictionary that Jonathan does not mention in his '14 Possessed nouns.doc'.
  # See my email of 10 Sept 06 10:22 AM.

