#!/usr/bin/env python
## -*- coding: utf-8 -*-
#
#Convert a tokenized text file to an XML parse file, in the format expected
# for the DOE project.
#
# Arguments: -p <ParsesFile>
#            -u (=> UTF-8)
# The first argument is obligatory, <ParsesFile> is a file as output by MergeByUR.py.
# The second argument is optional (and if not given,
# the input is assumed to be non-UTF8, presumably ISO).
# Input and output are stdin and stdout.  Input consists of a token stream, one
# token per line.
##FIX: The input will eventually include xml tags like <spanish>foo</spanish>
#      and ellipsis.


import sys
import getopt
import codecs   #For UTF-8

global g_bUTF8    #True if we're reading and writing UTF-8


def OpenInput():
    """Open the two input files (output is to stdout).
       Return the input and output streams, and the parse file stream."""

    #Default is non-Unicode:
    global g_bUTF8
    g_bUTF8 = False

    #Look for command line arg:
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'p:u:')
    except getopt.GetoptError:
        print 'Usage: Text2Parses.py -p <ParsesFile> (-u)'
        sys.exit(1)

    for o in opts:
        if o[0] == '-p':
            sParsesFile = o[1]
        elif o[0] == '-u':
            g_bUTF8 = True
            (UTF8Encode, UTF8Decode, UTF8Reader, UTF8Writer) = codecs.lookup('utf-8')
            #We don't use the UTF8Encode or UTF8Decode, but they are returned by codecs.lookup()

    #Input:
    try:
        if g_bUTF8:
            InStream = UTF8Reader(sys.stdin)
        else:
            InStream = sys.stdin
    except IOError:
        print "Error: Cannot open stdin"
        sys.exit(1)

    #Output:
    try:
        if g_bUTF8:
            OutStream = UTF8Writer(sys.stdout)
        else:
            OutStream = sys.stdout
    except IOError:
        print "Error: Cannot open stdout"
        sys.exit(1)

    #Parses file:
    try:
        if g_bUTF8:
            ParsesStream = UTF8Writer(open(sParsesFile, 'r'))
        else:
            ParsesStream = open(sParsesFile, 'r')
    except IOError:
        print "Error: Cannot open file %s" %sParsesFile
        sys.exit(1)


    return (InStream, OutStream, ParsesStream)


def Parses2Dictionary(ParsesStream):
    """Read all the unique parses into a Python dictionary struc, and return that dictionary.
       The dictionary is indexed by surface form, and the values consist of a list of
       2-tuples, each 2-tuple consisting of an underlying form and a gloss string."""
    dictParses = {}
    for sParseLine in ParsesStream.readlines():
        sParseLine = sParseLine.strip()
        try:
            (sSurfaceForm, sUnderlyingForm, sGloss) = sParseLine.split('\t')
            if dictParses.has_key(sSurfaceForm):
                #We've already run across sSurfaceForm, so it must parse ambiguously;
                # add this new parse:
                dictParses[sSurfaceForm].append((sUnderlyingForm, sGloss))
            else:
                #This is a new surface form:
                dictParses[sSurfaceForm] = [(sUnderlyingForm, sGloss)]
        except:
            sys.stderr.write("Unable to break line in parses file: '%s'\n" %sParseLine)
    return dictParses


#--------------MAIN--------------

#--------------Process records:
#Init the pipeline:
(InStream, OutStream, ParsesStream) = OpenInput()

#Read all the parses into a Python dictionary struc, indexed by surface form:
dictParses = Parses2Dictionary(ParsesStream)
ParsesStream.close()

#Now transform the input stream of tokens into an output XML stream of parses:
if g_bUTF8:
    OutStream.write('<?xml version="1.0" encoding="UTF-8"?>\n')
else:
    OutStream.write('<?xml version="1.0" encoding="ISO-8859-1"?>\n')
OutStream.write('<parserOutput>\n')
for sInToken in InStream.readlines():
    sInToken = sInToken.strip()
    OutStream.write('\t<parses form="%s">\n' %sInToken)
    if dictParses.has_key(sInToken):
        for (sUnderlyingForm, sGloss) in dictParses[sInToken]:
            OutStream.write('\t\t<parse morphemes="%s" glosses="%s" />\n' %(sUnderlyingForm, sGloss))
    OutStream.write('\t</parses>\n')
OutStream.write('<parserOutput>\n')
InStream.close()
OutStream.close()

