#!/bin/env python
# Convert the XML representation of one or more POSs, each with
# one or more paradigms, into HTML tables.
#
# Arguments:
#      -x <fname>  #(Obligatory) an XML file listing the POSs with paradigms
#      -h <fname>  #(Obligatory) An html file to output
#      -f <string> #Name of "feature set" of paradigms to use for columns
#                  # Name must appear as label of ??
#                  # (all other features will be used as rows)
#
# -----------Input format:
# XML file, as output by xfst running code produced by GenParadigms.py.
#
# -----------Output format:
# The immediate output consist of a series of xfst commands to generate the
# surface forms (if any) corresponding to each combination of feature structures,
# together with the XML tags.  If run through xfst (with a lexicon loaded and
# assigned to the var 'Lex'), this will give the following XML:
#    <?xml version="1.0" encoding="ISO-8859-1"?>
#    <!--Generated by GenParadigm.py from Sample.features 09-14-2004 07:13 AM-->
#    <?xml-stylesheet type="text/css" href="Paradigm.xsl"?>
#    <POS name="noun" stem="foo">
#        <Paradigm name="first person">
#           <Cell>
#               <Features> ... </Features>
#               <Wordforms>
#               (wordforms, one per line)
#               </Wordforms>
#           </Cell>
#        </Paradigm>
#        (additional paradigms)
#    </POS>
#    (additonal POSs)
## It would be nice to put the wordforms that xfst outputs inside individual XML
# tags.  Unfortunately, we can't easily control xfst's output in that way.  ,
# However the xfst 'echo' cmd produces lines with a leading space, whereas
# the generated words start in column 1, and contain no blanks.  So putting
# the generated words inside XML tags can be done by a 'sed' cmd like the ff.
# operating on the output:
#      	sed -e 's%^[^<> ][^<> ]*$$%       <Wordform form="&"/>%' \
# (The regex "[^<> ]" means anything but a '<', '>', or a space; the repetition
# with Kleene star is necessary because 'sed' lacks Kleene plus.)


import sys
import getopt
import string, re
import copy
import time     #To timestamp output file
import xml.dom.minidom
from sets import Set  #Used for debugging
from tempfile import mkdtemp
import os       #remove, rmdir


#------------------------------Globals---------------------------------
global g_bDebugging

#------------------------------Functions-------------------------------

